import axios from 'axios';
// import {notify} from '@/utils/notify'
import {toLogin} from "@/utils/permissions";
export class Interceptors {
  public instance: any

  constructor() {
    this.instance = axios.create({timeout: 1000 * 300})
    this.initInterceptors()
  }

  public getInterceptors() {
    return this.instance
  }

  public initInterceptors() {
    // this.instance.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded;charset=utf-8'
    /**
     * 请求拦截器
     * 每次请求前，如果存在token则在请求头中携带token
     */
    this.instance.interceptors.request.use(
      (config: any) => {
        if (!config.headers.Authorization) {
          const token = localStorage.getItem('ax_token');
          if (token) {
            config.headers.Authorization = token;
          } else {
            // @ts-ignore
            delete config.headers.Authorization
          }
        }
        return config;
      },
      (error: any) => {
      }
    );

    // 响应拦截器
    this.instance.interceptors.response.use(
      // 请求成功
      (res: any) => {
        if (res.headers.authorization) {
          localStorage.setItem('ax_token', res.headers.authorization);
        } else {
          if (res.data && res.data.code === 200 && res.data.token) {
            localStorage.setItem('ax_token', res.data.token);
          }
        }
        if (res.status === 200) {
          if(res.data.code === 200){
            return Promise.resolve(res.data);
          } else if (res.data.code === 401) {
            if(res.data.msg){
              //@ts-ignore
              // notify.warning(res.data.msg);
            }
            toLogin()
          } else {
            return Promise.resolve(res.data);
          }
        } else {
          this.errorHandle(res);
          return Promise.reject(res.data);
        }
      },
      // 请求失败
      (error: { response: any; }) => {
        const { response } = error;
        if (response) {
          // 请求已发出，但是不在2xx的范围
          this.errorHandle(response);
          return Promise.reject(response.data);
        } else {
          //@ts-ignore
          // notify.warning('网络连接异常,请稍后再试!');
          // 抛出报错信息，在页面里需要接收
          return Promise.reject(error);
        }
      });
  }

  private errorHandle(res: any) {
    console.error('错误接口：' + res.data.path)
    // 状态码判断
    switch (res.status) {
      case 401:
        break;
      case 403:
        break;
      case 404:
        //@ts-ignore
        // notify.warning('请求的资源不存在');
        break;
      default:
        //@ts-ignore
        // notify.warning('连接错误');
    }
  }
}
