import {createRouter, createWebHistory} from 'vue-router'
import staticRouter from './modules/static'
import store from '@/store/index'
import Temp404 from '@/views/global/temp/404/.vue'
import * as api from '@/api/index'
import {toLogin} from "@/utils/permissions";

const routes = [
    ...staticRouter,
    { path: '/:pathMatch(.*)*', name: 'NotFound', component: Temp404 }
]

const router = createRouter({
    history: createWebHistory(),
    routes,
});

router.beforeEach((to, from , next) => {
    if (to.meta?.noAuth) {
        if (to.path === '/login' && localStorage.getItem('ax_token')) {
            next({
                path: '/'
            })
        } else {
            next()
        }
    } else {
        if (to.query.axToken) {
            localStorage.removeItem('navigations')
            localStorage.removeItem('menuExpend')
            localStorage.setItem("ax_token", <string>to.query.axToken);
            setTimeout(() => {//虽然不知道为什么要加宏任务，但是我不敢改
                delete to.query.axToken
                let url = `${window.location.origin + window.location.pathname}?`
                for (let queryKey in to.query) {
                    url += `${queryKey}=${to.query[queryKey]}&`
                }
                window.location.replace(url.slice(0, url.length - 1));
                getInit(to, next)
            }, 0);
        } else if (localStorage.getItem("ax_token")) {
            getInit(to, next)
        } else {
            toLogin()
        }
    }
})

const getInit = (to: any, next: any) => {
    document.title = to?.meta?.title || (import.meta as any).env.VITE_PROJECT_TITLE
    if (to.path === '/') {
        const firstRoute = store.state.menu.allRouters[0]
        next({
            path: firstRoute.redirect || firstRoute.path
        })
    } else {
        next()
    }
}

export const initMainRouter = async () => {
    // 获取当前页面的 URL
    const urlParams = new URLSearchParams(window.location.search);
    // 获取名为 axToken 的参数值
    const axToken = urlParams.get('axToken');
    if (axToken) {
        localStorage.setItem("ax_token", axToken);
    }
    if (localStorage.getItem("ax_token") && !['/path', '/find'].includes(location.pathname)) {
        await Promise.all([
            store.dispatch('app/LOAD_USER_INFO')
        ]).then(async ([r, u]) => {

        })
    }

}
export default router;
